library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-- Count the number of switches
-- Count in groups of 4 and condense the output such
-- that finally a binary representation is there for
-- the number of switches. 

entity decide_smart is 
  port (
    switch_i:   in std_ulogic_vector(13 downto 0);
    led_o:      out std_ulogic
  );
end; 

architecture rtl of decide_smart is

  component addcomb is 
  port (
    switch_i:   in std_ulogic_vector(3 downto 0);
    sum_o:      out unsigned(2 downto 0)
  );
  end component;
  
  signal i16 : std_ulogic_vector(15 downto 0);
  
  signal l1_s1 : std_ulogic_vector(3 downto 0);
  signal l1_s2 : std_ulogic_vector(3 downto 0);
  signal l1_s4 : std_ulogic_vector(3 downto 0);
  signal l2_s4 : std_ulogic_vector(3 downto 0);
  signal l2_s2 : std_ulogic_vector(1 downto 0); 
  signal l2_s8 : std_ulogic_vector(3 downto 0);
  signal s16: std_ulogic_vector(2 downto 0);
  
  signal s1,s2,s4,s8, dummy : std_ulogic;
    
begin
  
  i16(13 downto 0) <= switch_i;
  i16(15 downto 14) <= "01"; 

  maj1 : for i in 0 to 3 generate
  begin
      addcombi : component addcomb
        port map(switch_i => i16((4*i+3) downto 4*i), sum_o(2) => l1_s4(i), sum_o(1) => l1_s2(i), sum_o(0) => l1_s1(i)); 
  end generate maj1;
  
  addcomb_l2_4 : addcomb
    port map (
      switch_i => l1_s4(3 downto 0),
      sum_o(2) => s16(0),
      sum_o(1) => l2_s8(0),
      sum_o(0) => l2_s4(0));
      
  addcomb_l2_2 : addcomb
    port map (
      switch_i => l1_s2(3 downto 0),
      sum_o(2) => l2_s8(1),
      sum_o(1) => l2_s4(1),
      sum_o(0) => l2_s2(0));
      
  addcomb_l2_1 : addcomb
    port map (
      switch_i => l1_s1(3 downto 0),
      sum_o(2) => l2_s4(2),
      sum_o(1) => l2_s2(1),
      sum_o(0) => s1);
      
  l2_s4(3) <= l2_s2(1) and l2_s2(0);
  s2 <= l2_s2(1) xor l2_s2(0);
       
  addcomb_l3_4 : addcomb
    port map (
      switch_i => l2_s4(3 downto 0),
      sum_o(2) => s16(1),
      sum_o(1) => l2_s8(2),
      sum_o(0) => s4);
          
  l2_s8(3) <= '0'; -- dummy
          
  addcomb_l3_8 : addcomb
    port map (
      switch_i => l2_s8(3 downto 0),
      sum_o(2) => dummy,
      sum_o(1) => s16(2),
      sum_o(0) => s8);
           
  led_o <= (s16(2) or s16(1) or s16(0)) or (s8 and (s4 or s2 or s1)) ;  
   			   
end; -- architecture


